/************************************
  REVISION LOG ENTRY
  Revision By: Mihai Filimon
  Revised on 9/10/98 2:32:48 PM
  Comments: TranslateWindowStyle.cpp: implementation of the CTranslateWindowStyle class.
 ************************************/

#include "stdafx.h"
#include "TranslateWindowStyle.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SMapKnownWindows CTranslatorWindowStyle::m_strWindowsClasses;
static const CString sSeparator(_T(" | "));

// Function name	: CTranslatorWindowStyle::CTranslatorWindowStyle
// Description	    : Default constructor
// Return type		: 
CTranslatorWindowStyle::CTranslatorWindowStyle()
{
	m_nStyle = 0x0;
	m_nStyle = 0x0;
	m_nLastStyle = 0x0;
	m_mapLastStyles = NULL;
	m_mapUnknowStyles = NULL;
	m_pWnd = NULL;
}

// Function name	: CTranslatorWindowStyle::~CTranslatorWindowStyle
// Description	    : virtual Destructor
// Return type		: 
CTranslatorWindowStyle::~CTranslatorWindowStyle()
{
}

// Function name	: CTranslatorWindowStyle::SetStyle
// Description	    : Set the new nStyle styles for translate this to strings
// Return type		: DWORD 
// Argument         : DWORD nStyle
DWORD CTranslatorWindowStyle::SetStyle(DWORD nStyle)
{
	DWORD nResult = m_nStyle; 
	m_nStyle = nStyle;
	// Need for Translate function
	m_mapLastStyles = (MAP_STYLE*)allStyles;
	m_nLastStyle = m_nStyle;
	m_mapUnknowStyles = &m_strWindowsClasses.m_mapKnownWindowsStyle;
	return nResult;
}

// Function name	: CTranslatorWindowStyle::SetStyle
// Description	    : Call to set member variable equal with window pWnd style. The next call of
// Translate function will convert this value styles
// Return type		: DWORD 
// Argument         : CWnd* pWnd
DWORD CTranslatorWindowStyle::SetStyle(CWnd* pWnd, DWORD nStyle)
{
	m_pWnd = pWnd;
	return SetStyle(nStyle || !pWnd ? nStyle : pWnd->GetStyle());
}

// Function name	: CTranslatorWindowStyle::SetExStyle
// Description	    : Set the new ExStyle to convert into strings
// Return type		: DWORD 
// Argument         : DWORD nExStyle
DWORD CTranslatorWindowStyle::SetExStyle(DWORD nExStyle)
{
	DWORD nResult = m_nExStyle; 
	m_nExStyle = nExStyle;
	// Need for Translate function
	m_mapLastStyles = (MAP_STYLE*)allExStyles;
	m_nLastStyle = m_nExStyle;
	m_mapUnknowStyles = &m_strWindowsClasses.m_mapKnownWindowsStyleEx;
	return nResult;
}

// Function name	: CTranslatorWindowStyle::SetExStyle
// Description	    : Call to set member variable equal with window pWnd extended style. The next call of
// Translate function will convert these extended styles
// Return type		: DWORD 
// Argument         : CWnd* pWnd
DWORD CTranslatorWindowStyle::SetExStyle(CWnd* pWnd, DWORD nStyle)
{
	m_pWnd = pWnd;
	return SetExStyle(nStyle || !pWnd ? nStyle : pWnd->GetExStyle());
}

// Function name	: CTranslatorWindowStyle::SetClassStyle
// Description	    : Set class style
// Return type		: DWORD 
// Argument         : DWORD nClassStyle
DWORD CTranslatorWindowStyle::SetClassStyle(DWORD nClassStyle)
{
	DWORD nResult = m_nClassStyle; 
	m_nClassStyle = nClassStyle;
	// Need for Translate function
	m_mapLastStyles = (MAP_STYLE*)allClassStyles;
	m_nLastStyle = m_nClassStyle;
	m_mapUnknowStyles = &m_strWindowsClasses.m_mapKnownWindowsStyleClass;
	return nResult;
}

// Function name	: CTranslatorWindowStyle::SetClassStyle
// Description	    : Call to set member variable equal with window pWnd class style. The next call of
// Translate function will convert these class styles
// Return type		: DWORD 
// Argument         : DWORD nClassStyle
DWORD CTranslatorWindowStyle::SetClassStyle(CWnd* pWnd, DWORD nStyle)
{
	m_pWnd = pWnd;
	return SetClassStyle(nStyle || !pWnd ? nStyle : ::GetClassLong(pWnd->m_hWnd, GCL_STYLE));
}

// Function name	: CTranslatorWindowStyle::TranslateStyle
// Description	    : Translate function
// Return type		: CString 
// Argument         : MAP_STYLE* pMapStyles
// Argument         : DWORD nStyle
CString CTranslatorWindowStyle::TranslateStyle(MAP_STYLE* pMapStyles, DWORD nStyle, DWORD* pUnsolvedStyle)
{
	CString sStyle; sStyle.Empty();
	DWORD aStyle = 0x0;
	while (pMapStyles->m_nStyle != -1)
	{
		if ((nStyle & pMapStyles->m_nStyle) == pMapStyles->m_nStyle)
			if (!((aStyle & pMapStyles->m_nStyle) == pMapStyles->m_nStyle))
			{
				sStyle += pMapStyles->m_lpszStyle + sSeparator;
				aStyle |= pMapStyles->m_nStyle;
			}
		pMapStyles++;
	}
	if (int nLen = sStyle.GetLength())
		sStyle = sStyle.Left(nLen - sSeparator.GetLength());
	if (pUnsolvedStyle)
		*pUnsolvedStyle = nStyle & ~aStyle;
	return sStyle;
}

// Function name	: CTranslatorWindowStyle::TranslateUnknownStyle
// Description	    : 
// Return type		: CString 
// Argument         : CWnd * pWnd
CString CTranslatorWindowStyle::TranslateUnknownStyle(CWnd * pWnd, DWORD nStyle)
{
	CString sResult; sResult.Empty();
	if (nStyle)
	{
		DWORD unknownStyle = NULL;
		if (pWnd)
			if (::IsWindow(pWnd->m_hWnd))
			{
				void* ptrArrays = NULL;
				if (m_mapUnknowStyles->Lookup(GetWindowClassName(pWnd), ptrArrays))
				{
					sResult = TranslateStyle((MAP_STYLE*)ptrArrays, nStyle, &unknownStyle) + sSeparator;
					nStyle = unknownStyle;
				}
			}
		if (nStyle)
		{
			CString text;
			text.Format(_T("0x%04X"), nStyle);
			sResult += text + sSeparator;
		}
		if (int nLen = sResult.GetLength())
			sResult = sResult.Left(nLen - sSeparator.GetLength());
	}
	return sResult;
}


// Function name	: CTranslatorWindowStyle::Translate
// Description	    : Call to translate into string the last style setted by Set... functions
// Return type		: CString 
CString CTranslatorWindowStyle::Translate()
{
	// Before call this function you need to call one of the Set... functions alike
	ASSERT (m_mapLastStyles != NULL);
	DWORD unknownStyle = NULL;
	CString kStyle = TranslateStyle(m_mapLastStyles, m_nLastStyle, &unknownStyle);
	CString kUnknownStyle = TranslateUnknownStyle(m_pWnd, unknownStyle);
	kStyle = kStyle + ((!kStyle.IsEmpty() && !kUnknownStyle.IsEmpty()) ? sSeparator : _T("")) + kUnknownStyle; 
	m_mapLastStyles = NULL;
	m_pWnd = NULL;
	return kStyle;
}

// Function name	: GetWindowClassName
// Description	    : Return the window class name of pWnd in lower case
// Return type		: CString 
// Argument         : CWnd* pWnd
CString CTranslatorWindowStyle::GetWindowClassName(CWnd* pWnd)
{
	ASSERT(::IsWindow(pWnd->m_hWnd));
	CString result;
	::GetClassName(pWnd->m_hWnd, result.GetBuffer(1024), 1024);
	result.ReleaseBuffer();
	result.MakeLower();
	return result;
}
